#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'max','min','sum';
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyAlternate 'xy_rotate_plus90','xy_rotate_minus90','rect_corners_sort';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

my $scale = 32;
my $width = 2*$scale;
my $height = 2*$scale;

my $pixels = '';
sub set_pixel {
  my ($x,$y) = @_;
  ### set_pixel(): "$x,$y"
  if ($x < 0 || $x >= $width || $y < 0 || $y >= $height) {
    die "set_pixel() out of range $x,$y";
  }
  vec($pixels, $x + $y*$width, 1) = 1;
}
sub clear_pixel {
  my ($x,$y) = @_;
  ### clear_pixel(): "$x,$y"
  die if $x < 0 || $x >= $width || $y < 0 || $y >= $height;
  vec($pixels, $x + $y*$width, 1) = 0;
}
sub get_pixel {
  my ($x,$y) = @_;
  if ($x < 0 || $x >= $width || $y < 0 || $y >= $height) {
    return 0;
  }
  return vec($pixels, $x + $y*$width, 1);
}

my $level = 10;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::AlternatePaper->new;

$Printwrap = 2;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $x_min = 10*$width;
my $x_max = 0;
my $y_min = 10*$height;
my $y_max = 0;

my $n_start = $n_hi/4;
my ($x1,$y1) = $path->n_to_xy($n_start);

foreach my $n ($n_start .. $n_hi) {
  next unless $n > 1;
  my ($x,$y) = $path->n_to_xy($n);
  my $sqrt = sqrt($n-1);
  $x /= $sqrt;
  $y /= $sqrt;
  $x *= $scale;
  $y *= $scale;
  set_pixel(int($x),int($y));

  $x_min = min($x_min, $x);
  $y_min = min($y_min, $y);
  $x_max = max($x_max, $x);
  $y_max = max($y_max, $y);
}

# set_pixel(3,3);
# set_pixel(3,4);

my $count_pixels = 0;
my $count_points = 0;
sub draw_pixels {
  my ($x,$y) = @_;
  ### draw_pixels(): "$x,$y"
  $count_pixels++;

  my @clear = [$x,$y];
  my $initial_dx = my $dx = -1;
  my $initial_dy = my $dy = 0;
  my $initial_x = $x;
  my $initial_y = $y;
  my $segments = MyLineSegments->new;

  do {
    my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
    my $nx = $x + $dx;
    my $ny = $y + $dy;
    ### at: "$x,$y  $dx,$dy consider $nx,$ny"
    if (get_pixel($nx,$ny)) {
      ## neighbour ...
      $x = $nx;
      $y = $ny;
      push @clear, [$x,$y];
      $count_pixels++;
      ($dx,$dy) = xy_rotate_minus90($dx,$dy);
    } else {
      ## no neighbour, side and turn ...
      $segments->add(2*$x+$dx + $rx, 2*$y+$dy + $ry,
                     2*$x+$dx - $rx, 2*$y+$dy - $ry);
      ($dx,$dy) = xy_rotate_plus90($dx,$dy);
    }
  } until ($x==$initial_x && $y==$initial_y
           && $dx==$initial_dx && $dy==$initial_dy);

  foreach my $point (@clear) {
    clear_pixel(@$point);
  }
  $Printwrap = 2;
  printwrap("\\draw ");
  $Printwrap += 2;
  $segments->printwrap (transform => sub {
                          my ($x,$y) = @_;
                          $x++; $y++;
                          return ($x/2/$scale, $y/2/$scale);
                        });;
  $count_points += $segments->lines_joined_count_points;
  printwrap(";\n");
}

foreach my $y (0 .. $height-1) {
  foreach my $x (0 .. $width-1) {
    if (get_pixel($x,$y)) {
      draw_pixels($x,$y);
    }
  }
}
print "  % $count_pixels pixels, drawn with $count_points points\n";
# 
# printf "  %% X %.4f %.4f  Y %.4f %.4f\n", $x_min, $x_max, $y_min, $y_max;
# printf "  %% GPC X %.4f %.4f  Y %.4f %.4f  M %.4f %.4f  L %.4f %.4f\n",
#   $points_x_min, $points_x_max, $points_y_min, $points_y_max,
#   $points_m_min, $points_m_max, $points_l_min, $points_l_max;

print "  % end generated\n";
exit 0;
