#!/usr/bin/perl -w

# Copyright 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min','max';

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'Gnum';
use MyLineSegments;
use MyPrintwrap ':all';


my $k = 6;

print "  % generated by tools/$FindBin::Script\n";

my $s_lo = 2**$k;
my $s_hi = 2**($k+1) - 1;

print "  \\newcommand{\\MyK}{$k}\n";
print "  \\newcommand{\\MySlo}{$s_lo}\n";
print "  \\newcommand{\\MyShi}{$s_hi}\n";
print "  \\begin{scope}[xscale=1/2^\\MyK]\n";

sub coord_to_str {
  my ($f) = @_;
  $f = sprintf '%.6f', $f;
  $f =~ s/\.?0*$//;       # trailing .000 or 1000
  $f =~ s/^0\././;        # leading 0.
  return $f;
}

my @s_decrease;
my @nodes;
my $f_min;
my $f_max;
$Printwrap = 2;
printwrap("\\draw[my line] ");
$Printwrap += 2;
my $prev_f;
{
  my $s = $s_lo - 1;
  $prev_f = 4*$s*$s/(Gnum($s)-1);
  $s = $s_lo;
  my $f = 4*$s*$s/(Gnum($s)-1);
  my $df = $prev_f - $f;
  $f = coord_to_str($f);
  $df = coord_to_str($df);
  printwrap("($s_lo - \\MyFirst, $f + $df*\\MyFirst)");
}
foreach my $s ($s_lo .. $s_hi) {
  my $f = 4*$s*$s/(Gnum($s)-1);
  if ($f < $prev_f) {
    push @s_decrease, $s-1;
  }

  $f_min = min($f_min//$f, $f);
  $f_max = max($f_max//$f, $f);

  $f = coord_to_str($f);
  printwrap("--($s,$f)");
  push @nodes, "$s,$f";

  $prev_f = $f;
}
{
  my $s = $s_hi + 1;
  my $f = 4*$s*$s/(Gnum($s)-1);
  my $df = $f - $prev_f;
  $prev_f = coord_to_str($prev_f);
  $df = coord_to_str($df);
  printwrap("--($s_hi + \\MyLast, $prev_f + $df*\\MyLast)");
}
printwrap(";\n");

$Printwrap = 2;
printwrap("\\path foreach \\p in {");
$Printwrap += 2;
foreach my $node (@nodes) { printwrap("($node),"); }
printwrap("}");
printwrap(" { \\p node[my dot] };\n");

print "  \\end{scope}\n";
print "% GP-DEFINE  gen_GRS_ratio_diagonal_steps_s_decrease",
  " = [",join(',',@s_decrease),"];\n";
print "  % end generated\n";
exit 0;
