#!/usr/bin/perl -w

# Copyright 2016, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min','max';
use Math::PlanePath::AlternatePaper;

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'chamfer','xy_rotate_plus90','xy_rotate_minus90';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 3;
my $option = $ARGV[0] || 'withtop';

print "  % generated by tools/$FindBin::Script $option\n";

my $path = Math::PlanePath::AlternatePaper->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my %seen;
$Printwrap = 2;
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  next if $seen{"$x,$y"}++;
  next if $option eq 'notop' && $x==2 && $y==2;
  my $text = (($x+$y)%2 == 0 ? 'y' : 'x');
  printwrap(" \\node at ($x,$y) {\$$text\$};");
  # (p$x-$y)
}
printwrap_newline();

$Printwrap = 2;
my ($prev_x,$prev_y);
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $x .= chamfer($dx);
  $y .= chamfer($dy);
  $x2 .= chamfer(-$dx);
  $y2 .= chamfer(-$dy);
  printwrap("\\draw[my arrow line] "
            . (defined $prev_x ? "($prev_x,$prev_y)--" : "")
            . "($x,$y)--($x2,$y2);");
  ($prev_x,$prev_y) = ($x2,$y2);
}
printwrap_newline();

$Printwrap = 2;
printwrap("\\draw[my expand line]");
$Printwrap += 2;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);

  my ($sx,$sy) = ($n%2==0
                  ? xy_rotate_plus90($dx,$dy)
                  : xy_rotate_minus90($dx,$dy));
  my $mx = $x + $dx/2 + $sx/2;
  my $my = $y + $dy/2 + $sy/2;
  my $mx1 = $mx . chamfer(-($dx/2 + $sx/2), '\\MyExpandChamfer');
  my $my1 = $my . chamfer(-($dy/2 + $sy/2), '\\MyExpandChamfer');
  my $mx2 = $mx . chamfer(-(-$dx/2 + $sx/2), '\\MyExpandChamfer');
  my $my2 = $my . chamfer(-(-$dy/2 + $sy/2), '\\MyExpandChamfer');

  $x .= chamfer($dx/2 + $sx/2, '\\MyExpandGap');
  $y .= chamfer($dy/2 + $sy/2, '\\MyExpandGap');
  $x2 .= chamfer(-$dx/2 + $sx/2, '\\MyExpandGap');
  $y2 .= chamfer(-$dy/2 + $sy/2, '\\MyExpandGap');

  printwrap(" ($x,$y)","--($mx1,$my1)","--($mx2,$my2)","--($x2,$y2)");
}
printwrap(";\n");

print "  % end generated\n";
exit 0;
