#!/usr/bin/perl -w

# Copyright 2014, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate ':all';
use MyPrintwrap ':all';


my $levels = 6;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::AlternatePaper->new;

foreach my $level (0 .. $levels) {
  $Printwrap = 2;
  printwrap("\\begin{scope}[shift=(pos-$level)]\n");
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  $n_hi -= 1;

  my $n_to_xy = sub {
    my ($n) = @_;
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    return ($x + $dx/2, $y + $dy/2);
  };

  $Printwrap = 4;
  {
    my ($x,$y) = $n_to_xy->($n_lo);
    printwrap("\\node (start-$level) at ($x,$y) [my dot];\n");
  }
  unless ($n_hi == $n_lo) {
    {
      my ($x,$y) = $n_to_xy->($n_hi);
      printwrap("\\node (end-$level) at ($x,$y) [my dot];\n");
    }
    printwrap("\\draw ");
    my $join = '';
    foreach my $n ($n_lo .. $n_hi) {
      my $semi = ($n == $n_hi ? ';' : '');

      my $p;
      if ($n == $n_lo) {
        $p = "(start-$level)";
      } elsif ($n == $n_hi) {
        $p = "(end-$level)";
      } else {
        my ($x,$y) = $n_to_xy->($n);
        $p = "($x,$y)";
      }
      printwrap("$join$p$semi");
      $join = '--';
    }
  }
  printwrap_newline();
  $Printwrap = 2;
  printwrap("\\end{scope}\n");
}

print "  % end generated\n";
exit 0;
