#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util 'min','max';

use FindBin;
use lib "$FindBin::Bin/../tools";
use MyAlternate 'dir';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $n_max = 512;
if (@ARGV) { $n_max = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";

my $segments = MyLineSegments->new;
my $prev = 0;
foreach my $n (0 .. $n_max) {
  my $value = dir($n);
  $segments->add($n,$prev, $n,$value);  # vertical from prev
  $segments->add($n,$value, $n+1,$value);
  $prev = $value;
}

my $scale = 10;

print "  \\begin{scope}[scale=1/",$n_max/$scale,"]\n";
$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap (transform => sub {
                        my ($x,$y) = @_;
                        return ($x/$scale, $y/$scale);
                      });
printwrap(";");
my $count = $segments->lines_joined_count_points;
printwrap(" % using $count points for n_max=$n_max\n");

print "  \\end{scope}\n";
print "  % end generated\n\n";
exit 0;
